/* gui.c
 *
 * Copyright (C) 2001 Jrgen Scheibengruber
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/*** the gui to gnomeradio */

#include <config.h>
#include <gnome.h>
#include <math.h>
#include "gui.h"
#include "tech.h"
#include "lirc.h"
#include "prefs.h"
#include "../pixmaps/digits.xpm"
#include "../pixmaps/signal.xpm"
#include "../pixmaps/stereo.xpm"
#include "../pixmaps/vol_up.xpm"
#include "../pixmaps/vol_down.xpm"
#include "../pixmaps/freq_up.xpm"
#include "../pixmaps/freq_down.xpm"

#define DIGIT_WIDTH 20
#define DIGIT_HEIGTH 30
#define SIGNAL_WIDTH 25
#define STEREO_WIDTH 35
#define SCAN_SPEED 20

static GtkWidget *drawing_area;
static GdkPixmap *digits, *signal, *stereo;
static GtkWidget *preset_menu, *menu;
static GtkWidget *freq_scale, *vol_scale;

static gint timeout_id;

void start_radio(gboolean restart, GtkWidget *app)
{
	gchar *buffer;
		
	if (restart)
		radio_stop();
	
	if (!radio_init(settings.device)) 
	{
		buffer = g_strdup_printf(_("Could not open device \"%s\" !\n\nCheck your Settings and make sure that no other\n"
		"program is using %s.\nMake also sure that you have read-access to it."), settings.device, settings.device);
		gnome_error_dialog_parented(buffer, GTK_WINDOW(app));
		g_free(buffer);
	}		
}

void start_mixer(gboolean restart, GtkWidget *app)
{
	gchar *buffer;
	gint res;
	
	if (restart)
		mixer_close();
	
	res = mixer_init(settings.mixer_dev, settings.mixer);
	if (res <1) 
	{
		if (res == -1)
			buffer = g_strdup_printf(_("Mixer source \"%s\" is not a valid source!"), settings.mixer);
		else 
			buffer = g_strdup_printf(_("Could not open \"%s\"!"), settings.mixer_dev);
		gnome_error_dialog_parented(buffer, GTK_WINDOW(app));
		g_free(buffer);
	}		
}

GList* get_mixer_recdev_list(void)
{
	int i;
	char **array, *dev;
	GList *result = NULL;
	
	array = mixer_get_rec_devices();
	
	if (!array)
		return NULL;
	
	i = 0;	
	dev = array[i];
	while (dev)
	{
		result = g_list_append(result, g_strdup(dev));
		free(dev);
		dev = array[++i];
	}			
	
	return result;
}

static gint expose_event_cb(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	GdkWindow *window;
	static GdkGC *gc;
	gint win_width, win_height;
	gint a, b, c, d, e;
	gint fl, rst, sign, ster;
		
	fl = (gint)(floor(adj->value/STEPS));
	rst = (gint)(gint)(adj->value*100/STEPS)%100;
	
	//g_print("Freq is: %.2f %i %i\n", adj->value/STEPS, fl, rst);
	a = fl-fl%100;
	b = (fl-a)-(fl-a)%10;
	c = fl-a-b; 
	e = rst%10;
	d = (rst-e)/10;

	a /= 100;
	b /= 10;
	
	window = drawing_area->window;

	gdk_window_get_size (window, &win_width, &win_height);
	if (!gc)
		gc = gdk_gc_new(window);
	
	gdk_draw_rectangle (window, gc, TRUE, 0, 0, -1, -1);

	win_width -= 5;
	
	sign = radio_getsignal();
	ster = radio_getstereo();
	
	if (sign > 3)
		sign = 3;
	if (sign < 0)
		sign = 0;
	
	if (a)
		gdk_draw_pixmap (window, gc, digits, a * DIGIT_WIDTH, 0, win_width - DIGIT_WIDTH*6, 5, DIGIT_WIDTH, DIGIT_HEIGTH);
	gdk_draw_pixmap (window, gc, digits, b * DIGIT_WIDTH, 0, win_width - DIGIT_WIDTH*5, 5, DIGIT_WIDTH, DIGIT_HEIGTH);
	gdk_draw_pixmap (window, gc, digits, c * DIGIT_WIDTH, 0, win_width - DIGIT_WIDTH*4, 5, DIGIT_WIDTH, DIGIT_HEIGTH);
	gdk_draw_pixmap (window, gc, digits, 10 * DIGIT_WIDTH, 0, win_width - DIGIT_WIDTH*3, 5, DIGIT_WIDTH, DIGIT_HEIGTH);
	gdk_draw_pixmap (window, gc, digits, d * DIGIT_WIDTH, 0, win_width - DIGIT_WIDTH*2, 5, DIGIT_WIDTH, DIGIT_HEIGTH);
	gdk_draw_pixmap (window, gc, digits, e * DIGIT_WIDTH, 0, win_width - DIGIT_WIDTH*1, 5, DIGIT_WIDTH, DIGIT_HEIGTH);
	gdk_draw_pixmap (window, gc, signal, sign * SIGNAL_WIDTH, 0, win_width - DIGIT_WIDTH*6-SIGNAL_WIDTH, 5, SIGNAL_WIDTH, DIGIT_HEIGTH);
	if (ster > -1)
		gdk_draw_pixmap (window, gc, stereo, ster * STEREO_WIDTH, 0, win_width - DIGIT_WIDTH*6-SIGNAL_WIDTH-STEREO_WIDTH, 5, STEREO_WIDTH, DIGIT_HEIGTH);
		
		
	
	//g_print("%i_%i_%i.%i%i\n", a, b, c, d, e);
	
	return 0;	
}
	
	
void exit_gnome_radio(void)
{
	if (settings.mute_on_exit)
	{
		radio_mute();
		radio_stop();
	}
	mixer_close();
	save_settings();
	gtk_main_quit();
}

static void adj_value_changed_cb(GtkAdjustment* data, gpointer window)
{
	gchar *buffer;

	expose_event_cb(NULL, NULL, NULL);
	
	buffer = g_strdup_printf(_("Gnomeradio - %.2f MHz"), adj->value/STEPS);
	gtk_window_set_title (GTK_WINDOW(window), buffer);
	g_free(buffer);
	
	buffer = g_strdup_printf(_("Frequency: %.2f MHz"), adj->value/STEPS);
	gtk_tooltips_set_tip(tooltips, freq_scale, buffer, NULL);
	g_free(buffer);

	radio_setfreq(adj->value/STEPS);
}

static gboolean freq_scale_focus_cb(GtkWidget *widget, GdkEventFocus *event, gpointer data)
{
	mom_ps = -1;
	preset_menu_set_item(mom_ps);
	return TRUE;
}
	
static void volume_value_changed_cb(GtkAdjustment* data, gpointer window)
{
	gchar *text;
	
	mixer_set_volume((gint)volume->value);
	
	text = g_strdup_printf(_("Volume: %d%%"), (gint)volume->value);
	gtk_tooltips_set_tip(tooltips, vol_scale, text, NULL);
	g_free(text);

}

static void stfw_button_clicked_cb(GtkButton *button, gpointer data)
{
	if (adj->value >= FREQ_MAX*STEPS)
		gtk_adjustment_set_value(adj, FREQ_MIN*STEPS);
	else
		gtk_adjustment_set_value(adj, adj->value+1);
	mom_ps = -1;
	preset_menu_set_item(mom_ps);
}

static void stbw_button_clicked_cb(GtkButton *button, gpointer data)
{
	if (adj->value <= FREQ_MIN*STEPS)
		gtk_adjustment_set_value(adj, FREQ_MAX*STEPS);
	else
		gtk_adjustment_set_value(adj, adj->value-1);
	mom_ps = -1;
	preset_menu_set_item(mom_ps);
}


static gboolean scan_freq(gpointer data)
{
	static gint signal, a, b, c;	
	static gint start, mom, max;
	gint dir = (gint)(data);
	
	if (!max)
		max = (FREQ_MAX - FREQ_MIN) * STEPS;

	if ((a + b + c > 8) || (start > max))	
	{
		start = mom = a = b = c = 0;
		radio_unmute();
		timeout_id = 0;
		return FALSE;
	}
	if (!mom)
	mom= adj->value;
	if (mom > FREQ_MAX*STEPS) 
		mom = FREQ_MIN*STEPS;
	else
	if (mom < FREQ_MIN*STEPS)
		mom = FREQ_MAX*STEPS;
	else	
		mom = mom + dir;
	start += 1;
	gtk_adjustment_set_value(adj, mom);
	signal = radio_getsignal();
	a = b;
	b = c;
	c = signal;
	return TRUE;
}

void scfw_button_clicked_cb(GtkButton *button, gpointer data)
{
	if (timeout_id)
		return;
	radio_mute();
	timeout_id = gtk_timeout_add(1000/SCAN_SPEED, (GtkFunction)scan_freq, (gpointer)1);	
	mom_ps = -1;
	preset_menu_set_item(mom_ps);

}

void scbw_button_clicked_cb(GtkButton *button, gpointer data)
{
	if (timeout_id)
		return;
	radio_mute();
	timeout_id = gtk_timeout_add(1000/SCAN_SPEED, (GtkFunction)scan_freq, (gpointer)(-1));	
	mom_ps = -1;
	preset_menu_set_item(mom_ps);
}

void preset_menu_set_item(gint i)
{
	gtk_option_menu_set_history(GTK_OPTION_MENU(preset_menu), i+1);
}

static void pref_item_activate_cb(GtkWidget *item, gpointer data)
{
	gint i = (gint)data;
	preset* ps = (preset*)g_list_nth_data(settings.presets,i);
	mom_ps = i;
	gtk_adjustment_set_value(adj, ps->freq*STEPS);
}

static void update_preset_menu(void)
{
	gint i, count;
	GtkWidget *item;
	preset *ps;
	
	if (menu)
		gtk_widget_destroy(menu);
	menu = gtk_menu_new();
	
	count = g_list_length(settings.presets);
	
	item = gtk_menu_item_new_with_label(_("manual"));
	gtk_menu_append(GTK_MENU(menu), item);
	
	for (i=0;i<count;i++)
	{
		ps = g_list_nth_data(settings.presets, i);
		item = gtk_menu_item_new_with_label(ps->name);
		gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(pref_item_activate_cb), (gpointer)i);
		gtk_menu_append(GTK_MENU(menu), item);
	}
	gtk_widget_show_all(menu);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(preset_menu), menu);
}

static void quit_button_clicked_cb(GtkButton *button, gpointer data)
{
	exit_gnome_radio();
}

static void prefs_button_clicked_cb(GtkButton *button, gpointer app)
{
	GtkWidget* dialog;
	gint choise;
	
	backup_settings();
	
	dialog = prefs_window();
	
	/* Michael Jochum <e9725005@stud3.tuwien.ac.at> proposed to not use gnome_dialog_set_parent()
	   but following instead. */
	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(app));
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
	
	/*gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));*/
	
	
	
	choise = gnome_dialog_run_and_close(GNOME_DIALOG(dialog));

	//choise ? g_print("You said cancel\n") : g_print("You said ok\n");
	if (!choise)
	{
		commit_settings(app);
		update_preset_menu();
	}
	else
		rollback_settings();
}

void mute_button_toggled_cb(GtkButton *button, gpointer data)
{
	static gboolean muted = FALSE;
	
	if (muted)
	{
		radio_unmute();
		muted = FALSE;
		gtk_tooltips_set_tip(tooltips, mute_button, _("Mute"), NULL);
	}
	else
	{
		radio_mute();
		muted = TRUE;
		gtk_tooltips_set_tip(tooltips, mute_button, _("Unmute"), NULL);
	}
}

static void about_button_clicked_cb(GtkButton *button, gpointer data)
{
	GtkWidget *about;
	const gchar *authors[] = {"Jrgen Scheibengruber mfcn@gmx.de", NULL};

	about = gnome_about_new ("Gnomeradio", VERSION, "Copyright 2001 Jrgen Scheibengruber",
                  (const gchar **) authors, _("Yet another Gnome FM-Tuner. But this one is the best ;-)\n"
                   "(Hopefully) works with all FM-Tunercards supported by video4linux."),
                   NULL);
 
	gtk_widget_show(about);

}

static gint delete_event_cb(GtkWidget* window, GdkEventAny* e, gpointer data)
{
	exit_gnome_radio();
	return TRUE;
}

GtkWidget* gnome_radio_gui(void)
{
	GtkWidget *app;
	GtkWidget *prefs_button, *quit_button, *scfw_button, *scbw_button, *stfw_button, *stbw_button, *about_button;
	GtkWidget *prefs_pixmap, *quit_pixmap, *scfw_pixmap, *scbw_pixmap, *stfw_pixmap, *stbw_pixmap, *about_pixmap, *mute_pixmap;
	GtkWidget *vol_up_pixmap, *vol_down_pixmap, *freq_up_pixmap, *freq_down_pixmap;
	GtkWidget *hbox1, *hbox2, *vbox, *menubox, *freq_vol_box;
	GtkWidget *hseparator1, *hseparator2, *vseparator1, *vseparator2, *vseparator3;
	GtkWidget *label;
	GtkWidget *frame1, *frame2;
	gchar *text;


	app = gnome_app_new(PACKAGE, _("Gnomeradio"));

	gtk_window_set_policy(GTK_WINDOW(app), FALSE, FALSE, FALSE);
	gtk_window_set_wmclass(GTK_WINDOW(app), "gnomeradio", "Gnomeradio");

	frame1 = gtk_frame_new(NULL);
	frame2 = gtk_frame_new(NULL);

	quit_pixmap = gnome_stock_pixmap_widget(app, GNOME_STOCK_PIXMAP_EXIT);
	prefs_pixmap = gnome_stock_pixmap_widget(app, GNOME_STOCK_PIXMAP_PREFERENCES);
	scfw_pixmap = gnome_stock_pixmap_widget(app, GNOME_STOCK_PIXMAP_LAST);
	scbw_pixmap = gnome_stock_pixmap_widget(app, GNOME_STOCK_PIXMAP_FIRST);
	stfw_pixmap = gnome_stock_pixmap_widget(app, GNOME_STOCK_PIXMAP_FORWARD);
	stbw_pixmap = gnome_stock_pixmap_widget(app, GNOME_STOCK_PIXMAP_BACK);
	about_pixmap = gnome_stock_pixmap_widget(app, GNOME_STOCK_PIXMAP_ABOUT);
	mute_pixmap = gnome_stock_pixmap_widget(app, GNOME_STOCK_PIXMAP_VOLUME);
	
	quit_button = gtk_button_new();
	prefs_button = gtk_button_new();
	scfw_button = gtk_button_new();
	scbw_button = gtk_button_new();
	stfw_button = gtk_button_new();
	stbw_button = gtk_button_new();
	about_button = gtk_button_new();
	mute_button = gtk_toggle_button_new();

	gtk_container_add(GTK_CONTAINER(quit_button), quit_pixmap);
	gtk_container_add(GTK_CONTAINER(prefs_button), prefs_pixmap);
	gtk_container_add(GTK_CONTAINER(scfw_button), scfw_pixmap);
	gtk_container_add(GTK_CONTAINER(scbw_button), scbw_pixmap);
	gtk_container_add(GTK_CONTAINER(stfw_button), stfw_pixmap);
	gtk_container_add(GTK_CONTAINER(stbw_button), stbw_pixmap);
	gtk_container_add(GTK_CONTAINER(about_button), about_pixmap);
	gtk_container_add(GTK_CONTAINER(mute_button), mute_pixmap);

	gtk_widget_set_usize(about_button, 30, 30);
	gtk_widget_set_usize(quit_button, 30, 30);

	vbox = gtk_vbox_new(FALSE, 0);
	hbox1 = gtk_hbox_new(FALSE, 0);
	hbox2 = gtk_hbox_new(FALSE, 0);
	menubox = gtk_vbox_new(FALSE, 0);
	freq_vol_box = gtk_hbox_new(FALSE, 0);
	
	adj = GTK_ADJUSTMENT(gtk_adjustment_new(SUNSHINE*STEPS, FREQ_MIN*STEPS, FREQ_MAX*STEPS+1, 1, 5, 1));
	volume = GTK_ADJUSTMENT(gtk_adjustment_new(100, 0, 101, 1, 10, 1));
	
	preset_menu = gtk_option_menu_new();
	gtk_widget_set_usize(preset_menu, 75, 25);	
	label = gtk_label_new(_("Presets:"));
	
	freq_scale = gtk_hscale_new(adj);
	vol_scale = gtk_hscale_new(volume);
	
	vol_up_pixmap = gnome_pixmap_new_from_xpm_d(vol_up_xpm);
	vol_down_pixmap = gnome_pixmap_new_from_xpm_d(vol_down_xpm);
	freq_up_pixmap = gnome_pixmap_new_from_xpm_d(freq_up_xpm);
	freq_down_pixmap = gnome_pixmap_new_from_xpm_d(freq_down_xpm);

	gtk_widget_set_usize(freq_scale, 160, 10);

	gtk_widget_realize(app);
	drawing_area = gtk_drawing_area_new();
	digits = gdk_pixmap_create_from_xpm_d (app->window, NULL, NULL, digits_xpm);
	signal = gdk_pixmap_create_from_xpm_d (app->window, NULL, NULL, signal_xpm);
	stereo = gdk_pixmap_create_from_xpm_d (app->window, NULL, NULL, stereo_xpm);
	
	hseparator1 = gtk_hseparator_new();
	hseparator2 = gtk_hseparator_new();
	vseparator1 = gtk_vseparator_new();
	vseparator2 = gtk_vseparator_new();
	vseparator3 = gtk_vseparator_new();
	
	gtk_scale_set_digits(GTK_SCALE(freq_scale), 0);
	gtk_scale_set_draw_value(GTK_SCALE(freq_scale), FALSE);
	gtk_scale_set_digits(GTK_SCALE(vol_scale), 0);
	gtk_scale_set_draw_value(GTK_SCALE(vol_scale), FALSE);

	gtk_drawing_area_size(GTK_DRAWING_AREA(drawing_area), DIGIT_WIDTH*6+10+SIGNAL_WIDTH+STEREO_WIDTH, DIGIT_HEIGTH+10);
	
	//gtk_widget_set_usize(spinbutton, 60, 25);

	gtk_container_add(GTK_CONTAINER(frame2), drawing_area);

	gtk_box_pack_start(GTK_BOX(hbox2), scbw_button, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(hbox2), stbw_button, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(hbox2), stfw_button, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(hbox2), scfw_button, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(hbox2), vseparator1, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(hbox2), mute_button, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(hbox2), vseparator2, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(hbox2), about_button, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(hbox2), prefs_button, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(hbox2), quit_button, TRUE, TRUE, 3);

	gtk_box_pack_start(GTK_BOX(hbox1), frame2, FALSE, FALSE, 3);
	gtk_box_pack_start(GTK_BOX(hbox1), menubox, TRUE, TRUE, 3);
	
	gtk_box_pack_start(GTK_BOX(menubox), label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(menubox), preset_menu, TRUE, TRUE, 0);

	gtk_box_pack_start(GTK_BOX(freq_vol_box), freq_down_pixmap, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(freq_vol_box), freq_scale, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(freq_vol_box), freq_up_pixmap, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(freq_vol_box), vseparator3, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(freq_vol_box), vol_down_pixmap, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(freq_vol_box), vol_scale, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(freq_vol_box), vol_up_pixmap, TRUE, TRUE, 2);

	gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 4);
	gtk_box_pack_start(GTK_BOX(vbox), freq_vol_box, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hseparator1, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 4);
	//gtk_box_pack_start(GTK_BOX(vbox), hseparator2, FALSE, FALSE, 0);
	
	gtk_container_add(GTK_CONTAINER(frame1), vbox);
	
	gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);

	gtk_container_set_border_width(GTK_CONTAINER(frame1), 3);
	gtk_container_set_border_width(GTK_CONTAINER(frame2), 2);

	gnome_app_set_contents(GNOME_APP(app), frame1);

	//status = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER);

	//gnome_app_set_statusbar(GNOME_APP(app), status);

	tooltips = gtk_tooltips_new();

	gtk_signal_connect(GTK_OBJECT(app), "delete_event", GTK_SIGNAL_FUNC(delete_event_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(quit_button), "clicked", GTK_SIGNAL_FUNC(quit_button_clicked_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(adj), "value-changed", GTK_SIGNAL_FUNC(adj_value_changed_cb), (gpointer) app);
	gtk_signal_connect(GTK_OBJECT(freq_scale), "motion-notify-event", GTK_SIGNAL_FUNC(freq_scale_focus_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(volume), "value-changed", GTK_SIGNAL_FUNC(volume_value_changed_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(stfw_button), "clicked", GTK_SIGNAL_FUNC(stfw_button_clicked_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(stbw_button), "clicked", GTK_SIGNAL_FUNC(stbw_button_clicked_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(scfw_button), "clicked", GTK_SIGNAL_FUNC(scfw_button_clicked_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(scbw_button), "clicked", GTK_SIGNAL_FUNC(scbw_button_clicked_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(about_button), "clicked", GTK_SIGNAL_FUNC(about_button_clicked_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(mute_button), "toggled", GTK_SIGNAL_FUNC(mute_button_toggled_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(prefs_button), "clicked", GTK_SIGNAL_FUNC(prefs_button_clicked_cb), (gpointer) app);
	gtk_signal_connect(GTK_OBJECT(drawing_area), "expose-event", GTK_SIGNAL_FUNC(expose_event_cb), NULL);

	gtk_tooltips_set_tip(tooltips, scbw_button, _("Scan Backwards"), NULL);
	gtk_tooltips_set_tip(tooltips, scfw_button, _("Scan Forwards"), NULL);
	gtk_tooltips_set_tip(tooltips, stbw_button, _("0.05 MHz Backwards"), NULL);
	gtk_tooltips_set_tip(tooltips, stfw_button, _("0.05 MHz Forwards"), NULL);
	gtk_tooltips_set_tip(tooltips, about_button, _("About Gnomeradio"), NULL);
	gtk_tooltips_set_tip(tooltips, prefs_button, _("Edit your Preferences"), NULL);
	gtk_tooltips_set_tip(tooltips, mute_button, _("Mute"), NULL);
	gtk_tooltips_set_tip(tooltips, quit_button, _("Quit"), NULL);
	text = g_strdup_printf(_("Frequency: %.2f MHz"), adj->value/STEPS);
	gtk_tooltips_set_tip(tooltips, freq_scale, text, NULL);
	g_free(text);
	text = g_strdup_printf(_("Volume: %d%%"), (gint)volume->value);
	gtk_tooltips_set_tip(tooltips, vol_scale, text, NULL);
	g_free(text);
	
	return app;
}


int main(int argc, char* argv[])
{
	GtkWidget* app;

	bindtextdomain(PACKAGE, GNOMELOCALEDIR);  
	textdomain(PACKAGE);

	gnome_init(PACKAGE, VERSION, argc, argv);  

	/* Technical Stuff */
	
	/* Main app */

	app = gnome_radio_gui();

	gtk_widget_show_all(app);

	load_settings();
	update_preset_menu();
	//g_print("momps: %i\n", mom_ps);
	preset_menu_set_item(mom_ps);

	start_radio(FALSE, app);
	
#ifdef HAVE_LIRC
	if(!my_lirc_init())
		gnome_error_dialog_parented(_("Could not start lirc"), GTK_WINDOW(app));
	else
		start_lirc();
#endif
		
	start_mixer(FALSE, app);
	
	adj_value_changed_cb(NULL, (gpointer) app);
	volume_value_changed_cb(NULL, NULL);
	
	gtk_main();

#ifdef HAVE_LIRC	
	my_lirc_deinit();
#endif

	return 0;
}

