/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"

#ifdef HAVE_LIRC

#include "lirc.h"
#include "gui.h"

static int fd = -1, input_tag = 0;
static struct lirc_config *config = NULL;

static void execute_lirc_command (char *cmd)
{
	preset *ps;
	int tmp;
	
	if (strcasecmp (cmd, "tune up") == 0) 
	{
		scfw_button_clicked_cb(NULL, NULL);
	}
	else if (strcasecmp (cmd, "tune down") == 0) 
	{
		scbw_button_clicked_cb(NULL, NULL);
	}
	else if (strcasecmp (cmd, "volume up") == 0) 
	{
		gtk_adjustment_set_value(volume, (volume->value > 95) ? 100 : volume->value+5);
	}
	else if (strcasecmp (cmd, "volume down") == 0) 
	{
		gtk_adjustment_set_value(volume,(volume->value < 5) ? 0 : volume->value-5);
	}
	else if (strcasecmp (cmd, "mute") == 0)
	{
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mute_button), !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(mute_button)));
	}
	else if (strcasecmp (cmd, "tv") == 0)
	{
		exit_gnome_radio();
		gnome_execute_shell(NULL, "xawtv"); 
	}
	else if (strcasecmp (cmd, "quit") == 0)
	{
		exit_gnome_radio();
	}
	else if (strcasecmp (cmd, "preset up") == 0)
	{
		if (g_list_length(settings.presets))
		{
			mom_ps = (mom_ps + 1)%g_list_length(settings.presets);
			ps = g_list_nth_data(settings.presets, mom_ps);
			gtk_adjustment_set_value(adj, ps->freq*STEPS);
			preset_menu_set_item(mom_ps);
		}
	}
	else if (strcasecmp (cmd, "preset down") == 0)
	{
		if (g_list_length(settings.presets))
		{
			if (mom_ps <= 0)
				mom_ps = g_list_length(settings.presets);
			mom_ps--;
			//mom_ps = (mom_ps - 1)%g_list_length(settings.presets);
			ps = g_list_nth_data(settings.presets, mom_ps);
			gtk_adjustment_set_value(adj, ps->freq*STEPS);
			preset_menu_set_item(mom_ps);
		}
	}
	else if (strncasecmp (cmd, "preset ", 7) == 0) 
	{
		sscanf(cmd, "preset %i", &tmp);
		if (tmp < g_list_length(settings.presets))
		{
			ps = g_list_nth_data(settings.presets, tmp);
			gtk_adjustment_set_value(adj, ps->freq*STEPS);
			mom_ps = tmp;
			preset_menu_set_item(mom_ps);
		}
	}
	else
	{
    	printf ("unrecognized lirccmd: %s\n", cmd);
	}
}

int my_lirc_init(void)
{
	g_print("Trying to bring up lirc\n");
	if ((fd = lirc_init ("gnomeradio", 0)) <=0) 
	{
		perror("lirc_init");
		return 0;
	}
  
	if (lirc_readconfig (NULL, &config, NULL) != 0) 
	{
		perror("lirc_readconfig");
    	lirc_deinit ();
    	fd = -1;
    	return 0;
	}
  
	fcntl (fd, F_SETFL, O_NONBLOCK);
	fcntl (fd, F_SETFD, FD_CLOEXEC);
  
	return 1;
}

void my_lirc_deinit(void)
{
	if (fd <= 0)
		return;
	printf("Shutting down lirc\n");
	gdk_input_remove (input_tag);
	lirc_freeconfig(config);
	lirc_deinit ();
	input_tag = 0;
}	

static void lirc_has_data_cb(gpointer data, gint fd, GdkInputCondition condition)
{
	char *code, *cmd;
	int ret = -1;
  
	while (lirc_nextcode (&code) == 0 && code != NULL) 
	{
		ret = 0;
		while (lirc_code2char (config, code, &cmd) == 0 && cmd != NULL) 
		{
			printf ("lirc command: %s\n", cmd);
			execute_lirc_command(cmd);
    	}
	free (code);
	}

	// on LIRC error, shutdown added input
	if (ret == -1) 
	{
		g_print("An lirc error occured\n");
		gdk_input_remove (input_tag);
		lirc_freeconfig (config);
		config = NULL;
		lirc_deinit ();
	}
}

void start_lirc(void)
{

	if (input_tag)
		return;	

	if (fd<0)
		return;

	input_tag = gdk_input_add (fd, GDK_INPUT_READ, lirc_has_data_cb, NULL);
}

#endif

