/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"
#include "prefs.h"
#include "gui.h"

#define UN_NAMED "unnamed"

static GtkWidget *name_entry, *device_entry;
static GtkWidget *mixer_combo;
static GtkWidget *mute_on_exit_cb;
static GtkWidget *clist;
static GtkAdjustment *spin;
static gint selected_row = -1;

static gnomeradio_settings tmp_settings;

gint save_settings(void)
{
	gint i, count;
	gchar buffer[30];
	preset *ps;
	
	count = g_list_length(settings.presets);
	
	//g_print("Saving Settings with %i presets\n", count);
	
	gnome_config_set_string("gnomeradio/global/device", settings.device);
	gnome_config_set_string("gnomeradio/global/mixer", settings.mixer);
	gnome_config_set_string("gnomeradio/global/mixer_device", settings.mixer_dev);
	gnome_config_set_bool("gnomeradio/global/mute_on_exit", settings.mute_on_exit);
	gnome_config_set_float("gnomeradio/global/volume", volume->value);
	gnome_config_set_float("gnomeradio/global/last", adj->value/STEPS);
	gnome_config_set_float("gnomeradio/global/preset", mom_ps);
	gnome_config_set_int("gnomeradio/global/presets", count);
	
	for (i=0;i<count;i++)
	{
		ps = g_list_nth_data(settings.presets, i);
		sprintf(buffer, "gnomeradio/Preset%i/name", i);
		gnome_config_set_string(buffer, ps->name); 
		sprintf(buffer, "gnomeradio/Preset%i/freq", i);
		gnome_config_set_float(buffer, ps->freq); 
	}	

	gnome_config_sync();

	return 1;
}			

gint load_settings(void)
{
	gint i, count;
	gchar buffer[30];
	preset *ps;
	
	settings.presets = NULL;
		
	settings.device = gnome_config_get_string("gnomeradio/global/device=/dev/radio");
	settings.mixer = gnome_config_get_string("gnomeradio/global/mixer=line");
	settings.mixer_dev = gnome_config_get_string("gnomeradio/global/mixer_device=/dev/mixer");
	settings.mute_on_exit = gnome_config_get_bool("gnomeradio/global/mute_on_exit=TRUE");
	gtk_adjustment_set_value(volume, gnome_config_get_float("gnomeradio/global/volume=100"));
	gtk_adjustment_set_value(adj, gnome_config_get_float("gnomeradio/global/last=106.25")*STEPS);
	mom_ps = gnome_config_get_float("gnomeradio/global/preset=-1");
	count = gnome_config_get_int("gnomeradio/global/presets=0");
	
	for (i=0;i<count;i++)
	{
		ps = malloc(sizeof(preset));
		sprintf(buffer, "gnomeradio/Preset%i/name=unnamed", i);
		strncpy(ps->name, gnome_config_get_string(buffer), 20);
		ps->name[20] = '\0';
		sprintf(buffer, "gnomeradio/Preset%i/freq=87.50", i);
		ps->freq = gnome_config_get_float(buffer); 
		settings.presets = g_list_append(settings.presets, (gpointer)ps);	
	}	
	return 1;
}			
	
gint backup_settings(void)
{
	preset *ps, *tmp;
	gint i;
	
	//g_print("Making a backup of settings\n");
		
	tmp_settings.device = g_strdup(settings.device);
	tmp_settings.mixer = g_strdup(settings.mixer);
	tmp_settings.mute_on_exit = settings.mute_on_exit;
	tmp_settings.presets = NULL;
	for (i=0;i<g_list_length(settings.presets);i++)
	{
		ps = malloc(sizeof(preset));
		tmp = g_list_nth_data(settings.presets, i);
		strcpy(ps->name, tmp->name);
		ps->freq = tmp->freq;
		tmp_settings.presets = g_list_append(tmp_settings.presets, (gpointer) ps);
	}
	return 1;
}

gint commit_settings(gpointer app)
{
	preset  *tmp;
	gint i;
	gchar* ptr;
	
	//g_print("Committing Settings\n");
	
	ptr = settings.device;
	settings.device = tmp_settings.device;
	if (strcmp(ptr, tmp_settings.device))
		start_radio(TRUE, app);
	g_free(ptr);
	tmp_settings.device = NULL;
	ptr = settings.mixer;
	settings.mixer = tmp_settings.mixer;
	if (strcmp(ptr, tmp_settings.mixer))
		start_mixer(TRUE, app);
	g_free(ptr);
	tmp_settings.mixer = NULL;
	settings.mute_on_exit = tmp_settings.mute_on_exit;
	for (i=0;i<g_list_length(settings.presets);i++)
	{
		tmp = g_list_nth_data(settings.presets, i);
		g_free(tmp);
	}
	g_list_free(settings.presets);
	settings.presets = tmp_settings.presets;
	tmp_settings.presets = NULL;
	
	selected_row = -1;	
	return 1;
}

gint rollback_settings(void)
{
	preset *tmp;
	gint i;
	
	//g_print("Undo Changes\n");
	
	g_free(tmp_settings.device);
	g_free(tmp_settings.mixer);
	for (i=0;i<g_list_length(tmp_settings.presets);i++)
	{
		tmp = g_list_nth_data(tmp_settings.presets, i);
		g_free(tmp);
	}
	g_list_free(tmp_settings.presets);
	tmp_settings.presets = NULL;
	
	selected_row = -1;	
	return 1;
}
	
static gboolean freq_spin_button_press_event_cb(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	switch (event->button)
	{
		case 4: gtk_adjustment_set_value(spin, spin->value-0.05);
			break;
		case 5: gtk_adjustment_set_value(spin, spin->value+0.05);
			break;
		default:;
	}	

	return TRUE;
}	

static void mute_on_exit_toggled_cb(GtkWidget* widget, gpointer data)
{
	tmp_settings.mute_on_exit = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mute_on_exit_cb));
}	

static gboolean device_entry_focus_out_event_cb(GtkWidget *widget, GdkEventFocus *event, gpointer data)
{
	if (tmp_settings.device)
		g_free(tmp_settings.device);
	tmp_settings.device = g_strdup(gtk_entry_get_text(GTK_ENTRY(device_entry)));
	return TRUE;
}

static gboolean mixer_entry_focus_out_event_cb(GtkWidget *widget, GdkEventFocus *event, gpointer data)
{
	if (tmp_settings.mixer)
		g_free(tmp_settings.mixer);
	tmp_settings.mixer = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(mixer_combo)->entry)));
	return TRUE;
}

static void add_button_clicked_cb(GtkWidget *widget, gpointer data)
{
	gtk_clist_unselect_all(GTK_CLIST(clist));
	gtk_entry_set_text(GTK_ENTRY(name_entry), "");
	selected_row = -1;
}

static void del_button_clicked_cb(GtkWidget *widget, gpointer data)
{
	gint row;
	preset *ps;
	
	if (selected_row<0)
		return;
	
	row = selected_row;
	ps = g_list_nth_data(tmp_settings.presets, selected_row);
	g_assert(ps);	
	tmp_settings.presets = g_list_remove(tmp_settings.presets, (gpointer)ps);
	g_free(ps);

	gtk_clist_remove(GTK_CLIST(clist), selected_row);
	if (row >= g_list_length(tmp_settings.presets))
	{
		selected_row = -1;
		gtk_clist_unselect_all(GTK_CLIST(clist));
		gtk_entry_set_text(GTK_ENTRY(name_entry), "");
	}
	else
	{
		selected_row = row;
		gtk_clist_select_row(GTK_CLIST(clist), selected_row, 0);
	}
}

static void update_button_clicked_cb(GtkWidget *widget, gpointer data)
{
	gchar *text, *row_data[2];
	gint rows;
	GtkAdjustment *v_scb;
	preset *ps;
	
	rows = g_list_length(tmp_settings.presets);
	//g_print("Selected row: %i\n", selected_row);
	//g_print("list_length: %i\n", rows);
	g_assert(selected_row < rows); 

	text = gtk_entry_get_text(GTK_ENTRY(name_entry));
	if (strlen(text)<1)
	{
		g_print("\a");
		return;
	}
	
	if (selected_row < 0)
	{
		ps = malloc(sizeof(preset));
		strcpy(ps->name, text);
		ps->freq = spin->value;
		tmp_settings.presets = g_list_append(tmp_settings.presets, (gpointer) ps);
	}				
	else
	{
		ps = (preset*)g_list_nth_data(tmp_settings.presets, selected_row);
		strcpy(ps->name, text);
		ps->freq = spin->value;
	}
	row_data[0] = text;
	text = g_strdup_printf("%.2f", spin->value);
	row_data[1] = text;
	
	//g_print("%s : %s \n", row_data[0], row_data[1]);
	
	if (selected_row < 0)
	{
		gtk_clist_append(GTK_CLIST(clist), row_data); 
		//selected_row = rows;
		//gtk_clist_select_row(GTK_CLIST(clist), rows, 0);
		gtk_clist_unselect_all(GTK_CLIST(clist));
		gtk_entry_set_text(GTK_ENTRY(name_entry), "");
		selected_row = -1;
		v_scb = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(data));
		gtk_adjustment_set_value(v_scb, v_scb->upper-v_scb->page_size);
	}
	else
	{
		gtk_clist_set_text(GTK_CLIST(clist), selected_row, 0, row_data[0]);
		gtk_clist_set_text(GTK_CLIST(clist), selected_row, 1, row_data[1]);
	}
	g_free(text);
}

static gboolean clist_key_press_event_cb(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	if (event->keyval == GDK_Delete)
		del_button_clicked_cb(widget, user_data);
	
	return TRUE;
}		

static gboolean enter_key_press_event_cb(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	if (event->keyval == GDK_Return)
		update_button_clicked_cb(widget, user_data);
	
	return TRUE;
}		

static void clist_select_row_cb(GtkCList *widget, gint row, gint column, GdkEventButton *event, gpointer user_data)
{
	gchar* text;
	preset *ps;

	g_assert(row < g_list_length(tmp_settings.presets));

	selected_row = row;
	ps = (preset*)g_list_nth_data(tmp_settings.presets, row);
	text = ps->name;
	gtk_entry_set_text(GTK_ENTRY(name_entry), text);
	//freq_entry_set_value(ps->freq);
	//freq_entry_activate_cb(NULL, NULL);	
	gtk_adjustment_set_value(spin, ps->freq);
}	

static void clist_unselect_row_cb(GtkCList *widget, gint row, gint column, GdkEventButton *event, gpointer user_data)
{
	selected_row = -1;
}	

static void spin_value_changed_cb(GtkWidget *widget, gpointer data)
{
	gint value;

	if (spin->value > FREQ_MAX)
		gtk_adjustment_set_value(spin, FREQ_MIN);
	if (spin->value < FREQ_MIN)
		gtk_adjustment_set_value(spin, FREQ_MAX);

	value = (gint)rint(spin->value*100);
	//g_print("%f :: %d\n",spin->value, value);
	
	if (value%5)
	{
		value = (gint)rint(spin->value*10)*10;
		gtk_adjustment_set_value(spin, (gfloat)value/100);
	}	
	else 
	{
		gtk_adjustment_set_value(adj, value*STEPS/100);
		mom_ps = -1;
		preset_menu_set_item(mom_ps);
	}	
}

GtkWidget* prefs_window(void)
{
	GtkWidget *dialog;
	GtkWidget *frame1, *frame2;
	GtkWidget *misc_box, *preset_box, *hbox;
	GtkWidget *device_label, *device_box;
	GtkWidget *mixer_label, *mixer_box;
	GtkWidget *name_label, *freq_label, *freq_spin;
	GtkWidget *button_box, *entry_box, *freq_box;
	GtkWidget *add_button, *del_button, *update_button;
	GtkWidget *add_pixmap, *del_pixmap, *update_pixmap;
	GtkWidget *scrolled_window;
	GtkWidget *separator;
	GList *mixer_devs;
	gchar *data[2];
	gint i;
	preset* ps;
	
	dialog = gnome_dialog_new(_("Gnomeradio Settings"), GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
	
	frame1 = gtk_frame_new(_("Misc Settings"));
	frame2 = gtk_frame_new(_("Presets"));

	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), frame1, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), frame2, TRUE, TRUE, 0);

	misc_box = gtk_vbox_new(FALSE, 6);
	gtk_container_set_border_width(GTK_CONTAINER(misc_box), 8);
	gtk_container_add(GTK_CONTAINER(frame1), misc_box);
	
	device_box = gtk_hbox_new(FALSE, 10);
	device_label = gtk_label_new(_("Radio Device:"));
	gtk_misc_set_alignment(GTK_MISC(device_label), 0.0f, 0.5f);
	device_entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(device_entry), tmp_settings.device);
	gtk_box_pack_start(GTK_BOX(device_box), device_label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(device_box), device_entry, TRUE, TRUE, 0);

	mixer_box = gtk_hbox_new(FALSE, 10);
	mixer_label = gtk_label_new(_("Mixer Source:"));
	gtk_misc_set_alignment(GTK_MISC(mixer_label), 0.0f, 0.5f);
	mixer_combo = gtk_combo_new();
	mixer_devs = get_mixer_recdev_list();
	if (mixer_devs)
	{
		gtk_combo_set_popdown_strings(GTK_COMBO(mixer_combo), mixer_devs);
		g_list_foreach(mixer_devs, (GFunc)g_free, NULL);
		g_list_free(mixer_devs);
	}
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(mixer_combo)->entry), tmp_settings.mixer);
	gtk_box_pack_start(GTK_BOX(mixer_box), mixer_label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(mixer_box), mixer_combo, TRUE, TRUE, 0);

	mute_on_exit_cb = gtk_check_button_new_with_label(_("Mute on exit?"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mute_on_exit_cb), tmp_settings.mute_on_exit);

	gtk_box_pack_start(GTK_BOX(misc_box), device_box, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(misc_box), mixer_box, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(misc_box), mute_on_exit_cb, FALSE, FALSE, 2);

	preset_box = gtk_vbox_new(FALSE, 4);
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 8);
	gtk_container_add(GTK_CONTAINER(frame2), hbox);

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	clist = gtk_clist_new(2);
	gtk_container_add(GTK_CONTAINER(scrolled_window), clist);
	gtk_widget_set_usize(clist, 50, 80);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
	gtk_clist_set_column_width(GTK_CLIST(clist), 0, 120);
	//gtk_clist_set_column_width(GTK_CLIST(clist), 1, 40);

	entry_box = gtk_hbox_new(FALSE, 10);
	button_box = gtk_vbox_new(TRUE, 3);

	name_label = gtk_label_new(_("Name:"));	
	gtk_misc_set_alignment(GTK_MISC(name_label), 0.0f, 0.5f);
	name_entry = gtk_entry_new_with_max_length(20);
	//gtk_entry_set_text(GTK_ENTRY(name_entry), UN_NAMED);
	
	gtk_box_pack_start(GTK_BOX(entry_box), name_label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(entry_box), name_entry, TRUE, TRUE, 0);

	add_pixmap = gnome_stock_pixmap_widget(dialog, GNOME_STOCK_PIXMAP_NEW);
	del_pixmap = gnome_stock_pixmap_widget(dialog, GNOME_STOCK_PIXMAP_TRASH);
	update_pixmap = gnome_stock_pixmap_widget(dialog, GNOME_STOCK_PIXMAP_SAVE);
	
	add_button = gtk_button_new();
	del_button = gtk_button_new();
	update_button = gtk_button_new();

	separator = gtk_hseparator_new();
	
	gtk_container_add(GTK_CONTAINER(add_button), add_pixmap);
	gtk_container_add(GTK_CONTAINER(del_button), del_pixmap);
	gtk_container_add(GTK_CONTAINER(update_button), update_pixmap);

	gtk_box_pack_start(GTK_BOX(button_box), add_button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(button_box), del_button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(button_box), update_button, FALSE, FALSE, 0);
	
	freq_box = gtk_hbox_new(FALSE, 4);
	freq_label = gtk_label_new(_("Frequency:"));
	gtk_misc_set_alignment(GTK_MISC(freq_label), 0.0f, 0.5f);
	/*freq_entry = gtk_entry_new_with_max_length(6);
	freq_entry_set_value(adj->value/STEPS);
	gtk_widget_set_usize(freq_entry, 50, 23);
	freq_spin = my_spin();*/
	
	spin = GTK_ADJUSTMENT(gtk_adjustment_new(adj->value/STEPS, FREQ_MIN-0.05, FREQ_MAX+0.05, 0.05, 1, 1));
	freq_spin = gtk_spin_button_new(spin, 0.05, 2);
	//gtk_widget_set_usize(freq_spin, 120, 23);
	
	gtk_box_pack_start(GTK_BOX(freq_box), freq_label, TRUE, TRUE, 0);
	//gtk_box_pack_start(GTK_BOX(freq_box), freq_entry, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(freq_box), freq_spin, TRUE, TRUE, 0);

	gtk_box_pack_start(GTK_BOX(preset_box), scrolled_window, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(preset_box), separator, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(preset_box), entry_box, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(preset_box), freq_box, TRUE, TRUE, 2);
	//gtk_box_pack_start(GTK_BOX(preset_box), button_box, TRUE, TRUE, 2);

	gtk_box_pack_start(GTK_BOX(hbox), preset_box, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), button_box, TRUE, TRUE, 10);

	gtk_signal_connect(GTK_OBJECT(freq_spin), "button-press-event", GTK_SIGNAL_FUNC(freq_spin_button_press_event_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(device_entry), "focus-out-event", GTK_SIGNAL_FUNC(device_entry_focus_out_event_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(mixer_combo)->entry), "focus-out-event", GTK_SIGNAL_FUNC(mixer_entry_focus_out_event_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(mute_on_exit_cb), "toggled", GTK_SIGNAL_FUNC(mute_on_exit_toggled_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(add_button), "clicked", GTK_SIGNAL_FUNC(add_button_clicked_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(del_button), "clicked", GTK_SIGNAL_FUNC(del_button_clicked_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(update_button), "clicked", GTK_SIGNAL_FUNC(update_button_clicked_cb), (gpointer)scrolled_window);
	gtk_signal_connect(GTK_OBJECT(clist), "select-row", GTK_SIGNAL_FUNC(clist_select_row_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(clist), "unselect-row", GTK_SIGNAL_FUNC(clist_unselect_row_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(clist), "key-press-event", GTK_SIGNAL_FUNC(clist_key_press_event_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(freq_spin), "key-press-event", GTK_SIGNAL_FUNC(enter_key_press_event_cb), (gpointer)scrolled_window);
	gtk_signal_connect(GTK_OBJECT(name_entry), "key-press-event", GTK_SIGNAL_FUNC(enter_key_press_event_cb), (gpointer)scrolled_window);
	gtk_signal_connect(GTK_OBJECT(spin), "value-changed", GTK_SIGNAL_FUNC(spin_value_changed_cb), NULL);

		
	for (i=0;i<g_list_length(tmp_settings.presets);i++)
	{
		ps = g_list_nth_data(tmp_settings.presets, i);
		data[0] = ps->name;
		data[1] = g_strdup_printf("%0.2f", ps->freq);
		gtk_clist_append(GTK_CLIST(clist), data);
		g_free(data[1]);
	}

	gtk_tooltips_set_tip(tooltips, device_entry, _("Specify the radio-device (in most cases /dev/radio)"), NULL);
	gtk_tooltips_set_tip(tooltips, GTK_COMBO(mixer_combo)->entry, 
	_("Choose the mixer source (line, line1, etc.) that is able to control the volume of your radio"), NULL);
	gtk_tooltips_set_tip(tooltips, mute_on_exit_cb, _("If unchecked, gnomeradio won't mute after exiting"), NULL);
	gtk_tooltips_set_tip(tooltips, add_button, _("Start a new preset"), NULL);
	gtk_tooltips_set_tip(tooltips, del_button, _("Remove preset from List"), NULL);
	gtk_tooltips_set_tip(tooltips, update_button, _("Add the preset to the List"), NULL);

	gtk_widget_show_all(dialog);

	return dialog;
}	
	
